package jp.webedi.sfi.bizx.server.communication;

import jp.webedi.sfi.bizx.server.commons.*;
import jp.webedi.sfi.bizx.server.dac.base.ConnectionFactory;
import jp.webedi.sfi.bizx.server.dac.base.ConnectionPool;
import jp.webedi.sfi.bizx.server.adapter.*;
import java.util.ArrayList;
import java.util.Hashtable;

public class JXMSTransferSoap extends BizXBase {

    private final String className = new String(this.getClass().getName()
            .substring((this.getClass().getName().lastIndexOf(".") + 1)));

    /**
     * $$ PutDocument᥽å
     *
     * @param messageId
     * @param data
     * @param senderId
     * @param receiverId
     * @param formatType
     * @param documentType
     * @param compressType
     * @return OKtrueʣfalseʳSOAP㳰
     */
    public boolean putDocument(IItemInfo iteminfo) throws Exception {

        String methodName = "putDocument";
        boolean mrc = false;
        boolean rc = false;
        Agent agent = null;
        Trader trader = null;
        String[] params = null;

        String senderId = iteminfo.getSenderId();
        String receiverId = iteminfo.getReceiverId();
        String messageId = iteminfo.getMessageId();
        byte[] data = iteminfo.getData();
        String formatType = iteminfo.getFormatType();
        String documentType = iteminfo.getDocumentType();
        String compressType = iteminfo.getCompressType();

        try {

            // С
            LogMgr.setLogVersion(ItemInfoImpl.getLogVersion());

            // Agent֥ȼ
            agent = new Agent(ItemInfoImpl.getInitLogConfFilePath(),
                    Constant.SYSTEM_NAME, ItemInfoImpl.getInitConfFilePath(),
                    Constant.RPCTYPE_PUT);

            params = new String[] { ItemInfoImpl.getHostName(), senderId,
                    receiverId, messageId };

            agent.getLogMgr().putLog(senderId, receiverId, messageId,
                    className, methodName, Constant.BLANK,
                    LogMgr.getLogMessage(MessageCatalog.SI219, params));

            // Trader֥Ȥ
            trader = new Trader(agent, senderId, receiverId);

            // 
            trader.verify(formatType, documentType);
            // å
            mrc = trader.sendMessage(messageId, data, compressType);

            // åOKʤͤtrueˤ
            if (mrc) {
                params = new String[] { ItemInfoImpl.getHostName(), senderId,
                        receiverId, messageId };
                agent.getLogMgr().putLog(senderId, receiverId, messageId,
                        className, methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.SI220, params));
            } else {
                params = new String[] { ItemInfoImpl.getHostName(), senderId,
                        receiverId, messageId };
                agent.getLogMgr().putLog(senderId, receiverId, messageId,
                        className, methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.ST606, params));
            }
            rc = mrc;

        } catch (CustomException ex) {
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
            try {
                if (agent != null) {
                    if (agent.getIsKeepConn()) {
                        ConnectionFactory.getInstance().getConnectionPool()
                                .closeConnection(agent.getConnection());
                        agent.setIsKeepConn(false);
                    }
                }
            } catch (Exception ex) {
                try {
                    agent.getLogMgr().putExceptionLog(Constant.LOG_LEVEL_ERROR,
                            ex);
                } catch (Exception innerEx) {
                    throw innerEx;
                }
                throw ex;
            }

            System.gc();
        }

        return rc;

    }

    /**
     * $$ GetDocument᥽å
     *
     * @param messageId
     *            ʰor͡
     * @param data
     *            ͡
     * @param senderId
     *            ͡
     * @param receiverId
     *            ʰ
     * @param formatType
     *            ͡
     * @param documentType
     *            ͡
     * @param compressType
     *            ͡
     * @return OKtrueʣfalseʳSOAP㳰
     */
    public boolean getDocument(IItemInfo iteminfo) throws Exception {

        String methodName = "getDocument";
        boolean mrc = false;
        boolean rc = false;
        Agent agent = null;
        Trader trader = null;
        Message ms = null;
        String[] params = null;
        String receiverId = iteminfo.getReceiverId();
        String messageId = iteminfo.getMessageId();

        try {

            // С
            LogMgr.setLogVersion(ItemInfoImpl.getLogVersion());

            // Agent֥ȼ
            agent = new Agent(ItemInfoImpl.getInitLogConfFilePath(),
                    Constant.SYSTEM_NAME, ItemInfoImpl.getInitConfFilePath(),
                    Constant.RPCTYPE_GET);

            if (messageId == null) {
                params = new String[] { ItemInfoImpl.getHostName(), receiverId };
                agent.getLogMgr().putLog(null, receiverId, null, className,
                        methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.SI221, params));
            } else {
                params = new String[] { ItemInfoImpl.getHostName(), receiverId,
                        messageId };
                agent.getLogMgr().putLog(null, receiverId, messageId,
                        className, methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.SI226, params));
            }

            // Trader֥Ȥ
            trader = new Trader(agent, receiverId);

            // å
            if (messageId == null) {
                mrc = trader.isRemainedMessage();
            } else {
                mrc = trader.isRemainedMessage(messageId);
            }

            if (mrc) {
                ms = trader.receiveMessage();
                // SoapRPCͤ򥻥å
                iteminfo.setSenderId(ms.getSenderId());
                iteminfo.setMessageId(ms.getMessageId());
                iteminfo.setData(ms.getMessageData().getData());
                iteminfo.setFormatType(ms.getFormatTypeName());
                iteminfo.setDocumentType(ms.getDocumentTypeName());
                // iteminfo.setCompressType("");

                // 20091207 akagi ̥
                CompressType compressType_ = new CompressType(agent);
                boolean ret = compressType_.getCompressTypeInfo(ms
                        .getCompressTypeNo());
                //
                agent.getLogMgr().putLog(null, receiverId, null, className,
                        methodName, Constant.BLANK,
                        String.valueOf(ms.getCompressTypeNo()));
                agent.getLogMgr().putLog(null, receiverId, null, className,
                        methodName, Constant.BLANK,
                        String.valueOf(ret));
                //
                if (ret) {
                    iteminfo.setCompressType(compressType_.getMimeMediaType());
                }
            }

            // åOKʤͤtrueˤ
            if (mrc) {
                params = new String[] { ItemInfoImpl.getHostName(),
                        iteminfo.getSenderId(), receiverId,
                        iteminfo.getMessageId() };
                agent.getLogMgr().putLog(iteminfo.getSenderId(), receiverId,
                        iteminfo.getMessageId(), className, methodName,
                        Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.SI222, params));
            } else {
                if (messageId == null) {
                    params = new String[] { ItemInfoImpl.getHostName(),
                            receiverId };
                    agent.getLogMgr().putLog(null, receiverId, null, className,
                            methodName, Constant.BLANK,
                            LogMgr.getLogMessage(MessageCatalog.ST607, params));
                } else {
                    params = new String[] { ItemInfoImpl.getHostName(),
                            receiverId, messageId };
                    agent.getLogMgr().putLog(null, receiverId, messageId,
                            className, methodName, Constant.BLANK,
                            LogMgr.getLogMessage(MessageCatalog.ST612, params));
                }
            }
            rc = mrc;

        } catch (CustomException ex) {
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
            try {
                if (agent != null) {
                    if (agent.getIsKeepConn()) {
                        ConnectionFactory.getInstance().getConnectionPool()
                                .closeConnection(agent.getConnection());
                        agent.setIsKeepConn(false);
                    }
                }
            } catch (Exception ex) {
                try {
                    agent.getLogMgr().putExceptionLog(Constant.LOG_LEVEL_ERROR,
                            ex);
                } catch (Exception innerEx) {
                    throw innerEx;
                }
                throw ex;
            }
            System.gc();
        }

        return rc;

    }

    /**
     * $$ GetDocumentList᥽å
     *
     * @param iteminfo
     * @return
     * @throws Exception
     */
    public boolean getDocumentList(IItemInfo iteminfo) throws Exception {

        String methodName = "getDocumentList";
        boolean rc = false;
        ArrayList advanceNoticeList = null;
        Agent agent = null;
        Message ms = null;
        String[] params = null;
        String receiverId = iteminfo.getReceiverId();
        String[] msgIdList = null;

        try {

            // С
            LogMgr.setLogVersion(ItemInfoImpl.getLogVersion());

            // Agent֥ȼ
            agent = new Agent(ItemInfoImpl.getInitLogConfFilePath(),
                    Constant.SYSTEM_NAME, ItemInfoImpl.getInitConfFilePath(),
                    Constant.RPCTYPE_GET);

            params = new String[] { ItemInfoImpl.getHostName(), receiverId };
            agent.getLogMgr().putLog(null, receiverId, null, className,
                    methodName, Constant.BLANK,
                    LogMgr.getLogMessage(MessageCatalog.SI221, params));

            ms = new Message(agent);

            advanceNoticeList = ms.getAdvanceNoticeList(receiverId);
            /* Ver2.2.0б */
            if (advanceNoticeList == null || advanceNoticeList.isEmpty()) {
                iteminfo.setAdvanceNoticeList(null);
            } else {
                iteminfo.setAdvanceNoticeList(advanceNoticeList);
                if (advanceNoticeList.size() > 0) {
                    msgIdList = new String[advanceNoticeList.size()];
                    for (int i = 0; i < advanceNoticeList.size(); i++) {
                        msgIdList[i] = (String) ((Hashtable) advanceNoticeList
                                .get(i)).get("MESSAGE_ID");
                    }
                }
                iteminfo.setMessageIdList(msgIdList);
                rc = true;
            }

            // if ( advanceNoticeList == null || advanceNoticeList.isEmpty() ) {
            // iteminfo.setMessageIdList( null );
            // } else {
            // if ( advanceNoticeList.size() > 0 ) {
            // msgIdList = new String[advanceNoticeList.size()];
            // for( int i = 0; i < advanceNoticeList.size(); i++ ) {
            // msgIdList[i] =
            // (String)((Hashtable)advanceNoticeList.get(i)).get("MESSAGE_ID");
            // }
            // }
            // iteminfo.setMessageIdList( msgIdList );
            // rc = true;
            // }

        } catch (CustomException ex) {
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
            try {
                if (agent != null) {
                    if (agent.getIsKeepConn()) {
                        ConnectionFactory.getInstance().getConnectionPool()
                                .closeConnection(agent.getConnection());
                        agent.setIsKeepConn(false);
                    }
                }
            } catch (Exception ex) {
                try {
                    agent.getLogMgr().putExceptionLog(Constant.LOG_LEVEL_ERROR,
                            ex);
                } catch (Exception innerEx) {
                    throw innerEx;
                }
                throw ex;
            }
            System.gc();
        }

        return rc;

    }

    /**
     * $$ ConfirmDocument᥽å
     *
     * @param messageId
     * @param senderId
     * @param receiverId
     * @return ǧOKtrueʣǧfalseʳSOAP㳰
     */
    public boolean confirmDocument(IItemInfo iteminfo) throws Exception {

        String methodName = "confirmDocument";
        boolean mrc = false;
        boolean rc = false;
        Agent agent = null;
        Trader trader = null;
        String senderId = iteminfo.getSenderId();
        String receiverId = iteminfo.getReceiverId();
        String messageId = iteminfo.getMessageId();
        String[] params = null;

        try {

            // С
            LogMgr.setLogVersion(ItemInfoImpl.getLogVersion());

            // Agent֥ȼ
            agent = new Agent(ItemInfoImpl.getInitLogConfFilePath(),
                    Constant.SYSTEM_NAME, ItemInfoImpl.getInitConfFilePath(),
                    Constant.RPCTYPE_CONFIRM);

            params = new String[] { ItemInfoImpl.getHostName(), senderId,
                    receiverId, messageId };
            agent.getLogMgr().putLog(senderId, receiverId, messageId,
                    className, methodName, Constant.BLANK,
                    LogMgr.getLogMessage(MessageCatalog.SI223, params));

            // Trader֥Ȥ
            trader = new Trader(agent, senderId, receiverId);

            // ǧ
            mrc = trader.confirmMessage(messageId);

            // ǧOKʤͤtrueˤ
            if (mrc) {
                params = new String[] { ItemInfoImpl.getHostName(), senderId,
                        receiverId, messageId };
                agent.getLogMgr().putLog(senderId, receiverId, messageId,
                        className, methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.SI224, params));
            } else {
                params = new String[] { ItemInfoImpl.getHostName(), senderId,
                        receiverId, messageId };
                agent.getLogMgr().putLog(senderId, receiverId, messageId,
                        className, methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.ST608, params));
            }
            rc = mrc;

        } catch (CustomException ex) {
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
            try {
                if (agent != null) {
                    if (agent.getIsKeepConn()) {
                        ConnectionFactory.getInstance().getConnectionPool()
                                .closeConnection(agent.getConnection());
                        agent.setIsKeepConn(false);
                    }
                }
            } catch (Exception ex) {
                try {
                    agent.getLogMgr().putExceptionLog(Constant.LOG_LEVEL_ERROR,
                            ex);
                } catch (Exception innerEx) {
                    throw innerEx;
                }
                throw ex;
            }
            System.gc();
        }

        return rc;

    }

    /**
     * $$ CancelDocument᥽å
     *
     * @param Iteminfo
     * @return 
     */
    public boolean cancelDocument(IItemInfo iteminfo) throws Exception {
        String methodName = "cancelDocument";
        boolean mrc = false;
        boolean rc = false;
        Agent agent = null;
        String senderId = iteminfo.getSenderId();
        String messageId = iteminfo.getMessageId();
        String[] params = null;
        Message ms = null;

        try {

            // С
            LogMgr.setLogVersion(ItemInfoImpl.getLogVersion());

            // Agent֥ȼ
            agent = new Agent(ItemInfoImpl.getInitLogConfFilePath(),
                    Constant.SYSTEM_NAME, ItemInfoImpl.getInitConfFilePath(),
                    Constant.RPCTYPE_CANCEL);

            params = new String[] { ItemInfoImpl.getHostName(), senderId,
                    messageId };
            agent.getLogMgr().putLog(senderId, null, messageId, className,
                    methodName, Constant.BLANK,
                    LogMgr.getLogMessage(MessageCatalog.SI245, params));

            ms = new Message(agent);

            mrc = ms.updateToDeletedStatus(senderId, messageId);

            // äOKʤͤtrueˤ
            if (mrc) {
                params = new String[] { ItemInfoImpl.getHostName(), senderId,
                        messageId };
                agent.getLogMgr().putLog(senderId, null, messageId, className,
                        methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.SI246, params));
                // äNGʺʤˤΤȤͤfalseˤ
            } else {
                params = new String[] { ItemInfoImpl.getHostName(), senderId,
                        messageId };
                agent.getLogMgr().putLog(senderId, null, messageId, className,
                        methodName, Constant.BLANK,
                        LogMgr.getLogMessage(MessageCatalog.ST613, params));
            }
            rc = mrc;

        } catch (CustomException ex) {
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
            try {
                if (agent != null) {
                    if (agent.getIsKeepConn()) {
                        ConnectionFactory.getInstance().getConnectionPool()
                                .closeConnection(agent.getConnection());
                        agent.setIsKeepConn(false);
                    }
                }
            } catch (Exception ex) {
                try {
                    agent.getLogMgr().putExceptionLog(Constant.LOG_LEVEL_ERROR,
                            ex);
                } catch (Exception innerEx) {
                    throw innerEx;
                }
                throw ex;
            }
            System.gc();
        }

        return rc;
    }

    /**
     * $$ reloadConfigure᥽å
     *
     * @return
     * @throws Exception
     */
    public boolean reloadConfigure() throws Exception {
        String methodName = "reloadConfigure";
        Config config = null;
        boolean rc = false;
        try {
            // ꥪ֥Ȥμ
            config = Config.getInstance();
            config.clearProperties();
            if ((ConnectionPool) ConnectionFactory.getInstance()
                    .getConnectionPool() != null) {
                ((ConnectionPool) ConnectionFactory.getInstance()
                        .getConnectionPool()).destroy();
                ConnectionFactory.getInstance().setConnectionPool(null);
                Agent.setIsConnEstablishStatus(false);
            }
            rc = true;
        } catch (CustomException ex) {
            throw ex;
        } catch (Exception ex) {
            throw ex;
        } finally {
        }
        return rc;
    }

}
